* Initialization
global out "SET_OUTPUT_FOLDER"

* Note: addplot() option only available in binscatter v7.10 and later


**** Familiar ground: scatter & reg

* Load data
clear all
sysuse nlsw88


* Scatter plot
count if !missing(wage,tenure)
scatter wage tenure, msize(vsmall) note("N=`=r(N)'", ring(0) pos(5))
graph export "$out/univariate_scatter.eps", replace

* OLS reg
qui log using "$out/univariate_reg.log", replace text
reg wage tenure
qui log close

* Draw the OLS fit line on scatter
twoway (scatter wage tenure, msize(vsmall)) (lfit wage tenure, lc(red) lwidth(medthick)), ///
	legend(off) ///
	ytitle("hourly wage")
graph export "$out/univariate_scatter_fit.eps", replace


**** Step-by-step binscatter

local s=1

* Define fixed axis boundaries
sum wage
local ymax=r(max)

sum tenure
local xmax=r(max)

* Bare scatter
scatter wage tenure, msize(vsmall) ///
	xscale(range(0 `xmax')) ///
	yscale(range(0 `ymax'))
graph export "$out/step_by_step_`s'.eps", replace
local ++s

* Add xlines
_pctile tenure, nq(10)
scatter wage tenure, msize(vsmall) ///
	xscale(range(0 `xmax')) ///
	yscale(range(0 `ymax')) ///
	xline(`=r(r1)' `=r(r2)' `=r(r3)' `=r(r4)' `=r(r5)' `=r(r6)' `=r(r7)' `=r(r8)' `=r(r9)', extend)
graph export "$out/step_by_step_`s'.eps", replace
local ++s

* Compute means	
xtile xq=tenure, nq(10)
egen xmean=mean(tenure), by(xq)
egen ymean=mean(wage), by(xq)

* Put mean in quantiles
forvalues q=1/10 {
	_pctile tenure, nq(10)
	twoway (scatter wage tenure if xq>`q', msize(vsmall)) ///
		(scatter ymean xmean if xq<=`q', mc(navy)), ///
		legend(off) ///
		xtitle("job tenure (years)") ///
		ytitle("hourly wage") ///
		xlab(0(5)25) ///
		ylab(0(10)40) ///
		xscale(range(0 `xmax')) ///
		yscale(range(0 `ymax')) ///
		xline(`=r(r1)' `=r(r2)' `=r(r3)' `=r(r4)' `=r(r5)' `=r(r6)' `=r(r7)' `=r(r8)' `=r(r9)', extend)
	graph export "$out/step_by_step_`s'.eps", replace
	local ++s
}

* Plot without lines
twoway (scatter ymean xmean, mc(navy)), ///
	legend(off) ///
	xtitle("job tenure (years)") ///
	ytitle("hourly wage") ///
	xlab(0(5)25) ///
	ylab(0(10)40) ///
	xscale(range(0 `xmax')) ///
	yscale(range(0 `ymax'))
graph export "$out/step_by_step_`s'.eps", replace
local ++s
		
	
* Plot with regression line
twoway (scatter ymean xmean, mc(navy)) ///
	(lfit wage tenure), ///
	legend(off) ///
	xtitle("job tenure (years)") ///
	ytitle("hourly wage") ///
	xlab(0(5)25) ///
	ylab(0(10)40) ///
	xscale(range(0 `xmax')) ///
	yscale(range(0 `ymax'))
graph export "$out/step_by_step_`s'.eps", replace
local ++s

* Scatter with regression line
twoway (scatter wage tenure, msize(vsmall)) (lfit wage tenure, lc(maroon) lwidth(medthick)), ///
	legend(off) ///
	ytitle("hourly wage")
graph export "$out/step_by_step_scatter_fit.eps", replace

* Plot binscatter
binscatter wage tenure, nq(10)
graph export "$out/univariate_binscatter.eps", replace
local ++s


/***** binscatter: Informative about SEs *****/

set seed 9034694

* Low variance
clear
local obs 200
set obs `obs'

gen tenure=runiform()*20
gen e=rnormal(0,.2)
gen wage = 7 + 0.2*tenure + e

reg wage tenure
local coef=string(_b[tenure],"%9.3f")
local se=string(_se[tenure],"%9.3f")
local t=string(_b[tenure]/_se[tenure],"%9.3f")
local r2=string(e(r2),"%9.3f")

twoway (scatter wage tenure, msize(vsmall)), ///
	ylab(0(5)15) ///
	note(" {&epsilon} ~ N(0, 0.2)", ring(0) pos(7) size(huge))
graph export "$out/se_lowvar_scatter.eps", replace

/*
twoway (scatter wage tenure, msize(vsmall)), ///
	ylab(0(5)15) ///
	note(" N=`obs'", ring(0) pos(7) size(huge)) ///
	subtitle("R{superscript:2} = `r2' ", ring(0) pos(5) size(huge))
graph export "$out/se_lowvar_scatter_r2.eps", replace
*/

binscatter wage tenure, ylab(6(2)12) yscale(range(6 12.4))
graph export "$out/se_lowvar_binsc.eps", replace

binscatter wage tenure, ylab(6(2)12) yscale(range(6 12.4)) ///
	note(`"Coef = `coef' "' ///
			`"(`se')"' ///
	, ring(0) pos(5) size(huge))
graph export "$out/se_lowvar_binsc_coef.eps", replace



* High variance
clear
local obs 200
set obs `obs'

gen tenure=runiform()*20
gen e=rnormal(0,2)
gen wage = 7 + 0.2*tenure + e

reg wage tenure
local coef=string(_b[tenure],"%9.3f")
local se=string(_se[tenure],"%9.3f")
local t=string(_b[tenure]/_se[tenure],"%9.3f")
local r2=string(e(r2),"%9.3f")

twoway (scatter wage tenure, msize(vsmall)), ///
	ylab(0(5)15) ///
	note(" N=`obs'", ring(0) pos(7) size(huge)) ///
	note(" {&epsilon} ~ N(0, 2.0)", ring(0) pos(7) size(huge))
graph export "$out/se_highvar_scatter.eps", replace

/*
twoway (scatter wage tenure, msize(vsmall)), ///
	ylab(0(5)15) ///
	note(" N=`obs'", ring(0) pos(7) size(huge)) ///
	subtitle("R{superscript:2} = `r2' ", ring(0) pos(5) size(huge))
graph export "$out/se_lowvar_scatter_r2.eps", replace
*/

binscatter wage tenure, ylab(6(2)12) yscale(range(6 12.4))
graph export "$out/se_highvar_binsc.eps", replace

binscatter wage tenure, ylab(6(2)12) yscale(range(6 12.4)) ///
	note(`"Coef = `coef' "' ///
			`"(`se')"' ///
	, ring(0) pos(5) size(huge))
graph export "$out/se_highvar_binsc_coef.eps", replace




/***** Binscatters are not informative about R^2 *****/

* Low individual variance

set seed 3486710

clear
local obs 200
set obs `obs'

gen tenure=runiform()*20
gen e=rnormal(0,.2)
gen wage = 7 + 0.2*tenure + e

reg wage tenure
local coef=string(_b[tenure],"%9.3f")
local se=string(_se[tenure],"%9.3f")
local t=string(_b[tenure]/_se[tenure],"%9.3f")
local r2=string(e(r2),"%9.3f")

twoway (scatter wage tenure, msize(vsmall)), ///
	ylab(0(5)15) ///
	note(" N=`obs'", ring(0) pos(7) size(huge))
graph export "$out/r2_lowvar_scatter.eps", replace

twoway (scatter wage tenure, msize(vsmall)), ///
	ylab(0(5)15) ///
	note(" N=`obs'", ring(0) pos(7) size(huge)) ///
	subtitle("R{superscript:2} = `r2' ", ring(0) pos(5) size(huge))
graph export "$out/r2_lowvar_scatter_r2.eps", replace

binscatter wage tenure
graph export "$out/r2_lowvar_binsc.eps", replace

binscatter wage tenure, ///
	note(`"Coef = `coef' "' ///
			`"(`se')"' ///
	, ring(0) pos(5) size(huge))
graph export "$out/r2_lowvar_binsc_coef.eps", replace

/*
binscatter wage tenure, ///
	note(`"Coef = `coef' "' ///
			`"(`se')"' ///
			" " ///
			"R-sq = `r2' " ///
	, ring(0) pos(5) size(huge))
graph export "$out/r2_lowvar_binsc_coef_r2.eps", replace

qui log using "$out/r2_lowvar_reg.log", replace text
reg wage tenure
qui log close
*/


* High individual variance
clear
local obs 2000
set obs `obs'

gen tenure=runiform()*20
gen e=rnormal(0,1.25)
gen wage = 7 + 0.2*tenure + e

reg wage tenure
local coef=string(_b[tenure],"%9.3f")
local se=string(_se[tenure],"%9.3f")
local t=string(_b[tenure]/_se[tenure],"%9.3f")
local r2=string(e(r2),"%9.3f")

twoway (scatter wage tenure, msize(vsmall)), ///
	ylab(0(5)15) ///
	note(" N=`obs'", ring(0) pos(7) size(huge))
graph export "$out/r2_highvar_scatter.eps", replace

twoway (scatter wage tenure, msize(vsmall)), ///
	ylab(0(5)15) ///
	note(" N=`obs'", ring(0) pos(7) size(huge)) ///
	subtitle("R{superscript:2} = `r2' ", ring(0) pos(5) size(huge))
graph export "$out/r2_highvar_scatter_r2.eps", replace

binscatter wage tenure
graph export "$out/r2_highvar_binsc.eps", replace

binscatter wage tenure, ///
	note(`"Coef = `coef' "' ///
			`"(`se')"' ///
	, ring(0) pos(5) size(huge))
graph export "$out/r2_highvar_binsc_coef.eps", replace

/*
binscatter wage tenure, ///
	note(`"Coef = `coef' "' ///
			`"(`se')"' ///
			" " ///
			"R-sq = `r2' " ///
	, ring(0) pos(5) size(huge))
graph export "$out/r2_highvar_binsc_coef_r2.eps", replace


qui log using "$out/r2_highvar_reg.log", replace text
reg wage tenure
qui log close
*/


/***** binscatter: informative about functional form *****/

set seed 9206786

clear
local obs 500
set obs `obs'


gen tenure=0.1+_n/`obs'*15
gen e=rnormal(0,1)
gen wage=10 + log(tenure) + e


qui log using "$out/funcform_reg.log", replace text
reg wage tenure
qui log close


binscatter wage tenure, ///
	yscale(range(9 13.11))
graph export "$out/funcform_binsc.eps", replace


binscatter wage tenure, ///
	yscale(range(9 13.11)) ///
	addplot( (function y=10 + log(x), range(.375 14.625) lc(gs10) lpattern(dash)) )
graph export "$out/funcform_binsc_withtrue.eps", replace

binscatter wage tenure, ///
	yscale(range(9 13.11)) ///
	addplot( (function y=10 + log(x), range(.375 14.625) lc(gs10) lpattern(dash)) ) ///
	note("N=500", ring(0) pos(5) size(medium))
graph export "$out/funcform_binsc_withtrue_n500.eps", replace


* Consistent as N -> inf

clear
local obs 5000000
set obs `obs'

gen tenure=0.1+runiform()*15
gen e=rnormal(0,1)
gen wage=10 + log(tenure) + e


binscatter wage tenure in 1/5000, ///
	yscale(range(9 13.11)) ///
	addplot( (function y=10 + log(x), range(.375 14.625) lc(gs10) lpattern(dash)) ) ///
	note("N=5,000", ring(0) pos(5) size(medium))
graph export "$out/funcform_binsc_withtrue_n5000.eps", replace

binscatter wage tenure in 1/500000, ///
	yscale(range(9 13.11)) ///
	addplot( (function y=10 + log(x), range(.375 14.625) lc(gs10) lpattern(dash)) ) ///
	note("N=5,000,000", ring(0) pos(5)  size(medium))
graph export "$out/funcform_binsc_withtrue_n5000000.eps", replace




/***** How many bins? *****/

clear
set obs 500

gen x=runiform()*20
gen y=.

* Linear
replace y = x
qui sum y
replace y=(y-r(min))/(r(max)-r(min))*20

foreach nq in 5 20 {
	binscatter y x, ylab(0(5)20) line(none) nq(`nq') ///
		note("`nq' bins", ring(0) pos(5) size(large)) xscale(range(0 21))
	graph export "$out/nbins_linear_nq`nq'.eps", replace
}

binscatter y x, ylab(0(5)20) line(none) nq(5) ///
	note("5 bins", ring(0) pos(5) size(large)) xscale(range(0 21))
binscatter y x, ylab(0(5)20) line(none) ///
	note("20 bins", ring(0) pos(5) size(large)) xscale(range(0 21))

* Cubic
replace y = ((x-10))^3
sum y
replace y=(y-r(min))/(r(max)-r(min))*20

foreach nq in 5 20 {
	binscatter y x, ylab(0(5)20) line(none) nq(`nq') ///
		note("`nq' bins", ring(0) pos(5) size(large)) xscale(range(0 21))
	graph export "$out/nbins_cubic_nq`nq'.eps", replace
}

binscatter y x, ylab(0(5)20) line(none) nq(5) ///
	note("5 bins", ring(0) pos(5) size(large)) xscale(range(0 21))
binscatter y x, ylab(0(5)20) line(none) ///
	note("20 bins", ring(0) pos(5) size(large)) xscale(range(0 21))


* Sinusoidal
replace y = sin(x)
sum y
replace y=(y-r(min))/(r(max)-r(min))*20

foreach nq in 5 20 100 {
	binscatter y x, ylab(0(5)20) line(none) nq(`nq') ///
		note("`nq' bins", ring(0) pos(5) size(large)) xscale(range(0 21))
	graph export "$out/nbins_sin_nq`nq'.eps", replace
}

binscatter y x, ylab(0(5)20) line(none) nq(5) ///
	note("5 bins", ring(0) pos(5) size(large)) xscale(range(0 21))
binscatter y x, ylab(0(5)20) line(none) nq(20) ///
	note("20 bins", ring(0) pos(5) size(large)) xscale(range(0 21))
binscatter y x, ylab(0(5)20) line(none) nq(100) ///
	note("100 bins", ring(0) pos(5) size(large)) xscale(range(0 21))


/***** Multivariate *****/

* Load data
clear all
sysuse nlsw88
rename ttl_exp experience

* No controls
reg wage tenure
local coef=string(_b[tenure],"%9.2f")
local se=string(_se[tenure],"%9.2f")

binscatter wage tenure, ///
	note(`"Coef = `coef' "' ///
			`"(`se')"' ///
			" " ///
	, ring(0) pos(5) size(medium)) ///
	ylab(5/10) yscale(range(5 10.5))
graph export "$out/multivariate_nocontrols.eps", replace

* Controlling for total work experience
reg wage tenure experience
local coef=string(_b[tenure],"%9.2f")
local se=string(_se[tenure],"%9.2f")

binscatter wage tenure, controls(experience) ///
	note(`"Coef = `coef' "' ///
			`"(`se')"' ///
			" " ///
	, ring(0) pos(5) size(medium)) ///
	ylab(5/10) yscale(range(5 10.5))
graph export "$out/multivariate_experience.eps", replace



* Manually perform partitioned regression
reg wage experience
predict wage_r, residuals
sum wage
replace wage_r=wage_r+r(mean)

reg tenure experience
predict tenure_r, residuals
sum tenure
replace tenure_r=tenure_r+r(mean)

binscatter wage_r tenure_r
graph export "$out/multivariate_experience_manual.eps", replace




/***** By-variable *****/

* Load data
clear all
sysuse nlsw88
keep if inrange(age,35,44) & inrange(race,1,2)


binscatter wage age, by(race) ///
	ylab(4(2)10) xscale(range(34 44.25))
graph export "$out/by_race_nocontrols.eps", replace

binscatter wage age, by(race) absorb(occupation) ///
	ylab(4(2)10) xscale(range(34 44.25))
graph export "$out/by_race_absorb_occupation.eps", replace



/***** RD *****/

set seed 873426
clear all

set obs 2000

gen tenure=floor(runiform()*30)
gen e=rnormal(0,1.25)
gen wage = 7.5 + 1 * (tenure>=3) + 0.2*min(tenure,15) * (tenure>=3) + e

* RD

twoway (function y=7.5, range(0 3) lc(maroon)) (function y=8.5+0.2*x, range(3 12) lc(maroon)), ///
	graphregion(fcolor(white)) ///
	xlab(0(2)12) ///
	yscale(range(8 11.1)) ///
	legend(off) ///
	xtitle("tenure") ///
	ytitle("wage") ///
	xline(3, lc(gs8) lpattern(dash))
graph export "$out/rd_wagesched.eps", replace

binscatter wage tenure if inrange(tenure,0,12), discrete ///
	line(none) ///
	yscale(range(8 11.1)) ///
	xlab(0(2)12)
graph export "$out/rd_binsc_noline.eps", replace

binscatter wage tenure if inrange(tenure,0,12), discrete ///
	rd(2.5) ///
	yscale(range(8 11.1)) ///
	xlab(0(2)12)
graph export "$out/rd_binsc_rd.eps", replace


* RK


twoway (function y=7.5, range(0 3) lc(maroon)) ///
	(function y=8.5+0.2*x, range(3 15) lc(maroon)) ///
	(function y=11.5, range(15 25) lc(maroon)), ///
	graphregion(fcolor(white)) ///
	xlab(0(5)25) ///
	yscale(range(8 12.1)) ///
	legend(off) ///
	xtitle("tenure") ///
	ytitle("wage") ///
	xline(3, lc(gs8) lpattern(dash)) ///
	xline(15, lc(gs8) lpattern(dash))
graph export "$out/rk_wagesched.eps", replace


binscatter wage tenure if inrange(tenure,0,25), discrete ///
	line(none) ///
	yscale(range(8 12.1)) ///
	xlab(0(5)25)
graph export "$out/rk_binsc_noline.eps", replace

binscatter wage tenure if inrange(tenure,0,25), discrete ///
	rd(2.5 14.5) ///
	yscale(range(8 12.1)) ///
	xlab(0(5)25)
graph export "$out/rk_binsc_rd.eps", replace



/****** Event study *****/

set seed 873426
clear all

set obs `=500*9'

gen time=mod(_n-1,9)-4
gen id=ceil(_n/9)

bys id: gen temp=round(runiform()) if _n==1
egen layoff=max(temp), by(id)
drop temp

gen wage=runiform()*8+3 if time==-4
forvalues t=-3/-1 {
	replace wage=wage[_n-1]+rnormal(0.2) if time==`t'
}
replace wage=wage[_n-1]+rnormal(2) if time==0 & layoff==0
replace wage=wage[_n-1]+rnormal(-.5) if time==0 & layoff==1
forvalues t=1/4 {
	replace wage=wage[_n-1]+rnormal(0.2) if time==`t'
}

binscatter wage time, line(connect) xline(-0.5) xlab(-4/4) ///
	xtitle(Years Since Job Switch)
graph export "$out/eventstudy_binsc.eps"


binscatter wage time, line(connect) xline(-0.5) xlab(-4/4) by(layoff) ///
	xtitle(Years Since Job Switch)
graph export "$out/eventstudy_binsc_by_layoff.eps"

